from io import StringIO
import unittest
from pprint import pprint
from .tests import test_rfcnt


def run():
    stream = StringIO()
    runner = unittest.TextTestRunner(stream=stream)
    result = runner.run(unittest.makeSuite(test_rfcnt.TestRainflowCounting))
    print('Tests run ', result.testsRun)
    print('Errors ', result.errors)
    pprint(result.failures)
    stream.seek(0)
    print('Test output\n', stream.read())


if __name__ == '__main__':
    run()



import os
import numpy as np
import matplotlib.ticker as ticker
from .. import rfcnt, utils


def __get_script_path():
    return os.path.dirname(__file__)


def example_1():
    try:
        import pandas as pd
        import matplotlib.pyplot as plt
        from matplotlib.gridspec import GridSpec
    except ImportError as err:
        print("This example requires modules pandas` and `matplotlib`!")
        raise err

    data = pd.read_csv(
        os.path.join(__get_script_path(), "long_series.csv"), header=None)
    data = data.to_numpy().squeeze()

    class_count = 50
    class_range = data.max() - data.min()
    class_width = class_range / (class_count - 1)
    class_offset = data.min() - class_width / 2

    res = rfcnt.rfc(
        data, class_count=class_count,
        class_offset=class_offset,
        class_width=class_width,
        hysteresis=class_width,
        use_HCM=0,
        use_ASTM=0,
        #spread_damage=0,   # RFC_SD_HALF_23
        spread_damage=8,   # RFC_SD_TRANSIENT_23c
        #residual_method=0, # RFC_RES_NONE
        residual_method=7, # RFC_RES_REPEATED
        wl={"sd": 1e3, "nd": 1e7, "k": 5})

    fig = plt.figure(figsize=(14, 10))
    gs = GridSpec(nrows=3, ncols=2, width_ratios=[1, 2])

    # RF matrix
    ax1 = fig.add_subplot(gs[0, 0])
    im = ax1.imshow(res["rfm"], cmap="YlOrRd", aspect=0.7)
    cb = plt.colorbar(im, ax=ax1, label="Counts")
    cb.ax.tick_params(labelsize="x-small")
    ax1.yaxis.set_major_locator(ticker.MultipleLocator(2))
    ax1.xaxis.set_major_locator(ticker.MultipleLocator(2))
    ax1.xaxis.set_ticks_position("top")
    ax1.xaxis.set_label_position("top")
    ax1.tick_params(axis="x", labelsize="x-small", labelrotation=90)
    ax1.tick_params(axis="y", labelsize="x-small")
    plt.grid(which="both")
    plt.xlabel("Class # (to)")
    plt.ylabel("Class # (from)")

    # Range pairs
    r = utils.rpplot_prepare(sa=res["rp"][:, 0] / 2, counts=res["rp"][:, 1])
    ax2 = fig.add_subplot(gs[0, 1])
    # Stairs plot from right to left
    ax2.plot(
        r["counts"].cumsum(),
        r["sa"],
        ds="steps-post")
    plt.xscale("log")
    plt.ylim(bottom=0, top=2500)
    plt.xlim(left=0.9)
    plt.grid(which="both")
    plt.xlabel("N (log) [1]")
    plt.ylabel("$S_a$")

    # Damage history
    ax3 = fig.add_subplot(gs[1, :])
    ax3.plot(np.arange(len(res["dh"])), res["dh"].cumsum())
    plt.grid(which="both")
    plt.xlabel("Sample #")
    plt.ylabel("Damage (cumulative)")

    # Time series
    ax4 = fig.add_subplot(gs[2, :])
    ax4.plot(np.arange(len(data)), data)
    plt.grid(which="both")
    plt.xlabel("Sample #")
    plt.ylabel("Value")

    fig.tight_layout()
    plt.show()



import os
import sys
from importlib.util import spec_from_file_location, module_from_spec
from . import utils
from . import tests


def _load_spec_extension_module():
    """Select and load a suitable extension build"""
    from numpy import __version__ as np__version__
    from numpy.lib import NumpyVersion

    EXT_DIR = "_ext"

    # see `numpy/numpy/core/include/numpy/numpyconfig.h`
    npy_version = NumpyVersion(np__version__)
    if npy_version < '1.20.0':
        # API version 0x8
        infix = "npy_1_19_5"
    elif npy_version < '1.22.0':
        # API version 0xe
        infix = "npy_1_21_6"
    elif npy_version < '1.23.0':
        # API version 0xf
        infix = "npy_1_22_4"
    else:
        # API version 0x10
        infix = "npy_1_23_5"
        
    modulename = [file for file in os.listdir(os.path.join(__path__[0], EXT_DIR)) if infix in file]

    if len(modulename) == 1:
        spec = spec_from_file_location(".rfcnt", os.path.join(__path__[0], EXT_DIR, modulename[0]))
        module = module_from_spec(spec)
        sys.modules[__name__ + spec.name] = module 
        spec.loader.exec_module(module)
        del spec, module, npy_version, infix
    else:
        raise ImportError("No suitable build found for numpy %s!", np__version__)


# Try to import the extension module, compiled from sources.
# When this module is installed from a wheel, there are multiple 
# versions depending on the version of numpy installed.
# In latter case, the suitable version will be loaded and
# finally imported.
try:
    from . import rfcnt
except ImportError:
    _load_spec_extension_module()
    from . import rfcnt



import numpy as np


def rpplot_prepare(sa, counts):
    x = np.vstack((np.array(sa).flatten(), np.array(counts).flatten())).T
    i = np.argsort(x[:, 0])
    x = x[i[::-1], :]
    x = np.vstack(((x[0, 0], 0), x, (0, 0.01)))

    return {"sa": x[:, 0], "counts": x[:, 1]}



import os
import unittest
import numpy as np
from .. import rfcnt


class TestRainflowCounting(unittest.TestCase):

    def get_script_path(self):
        return os.path.dirname(__file__)

    def class_param(self, data, class_count):
        assert(class_count > 1)

        if len(data) == 0:
            class_width  = 1
            class_offset = 0
        else:
            class_width  = (data.max() - data.min()) / (class_count - 1)
            class_width  = np.ceil(class_width * 100) / 100
            class_offset = np.floor((data.min() - class_width / 2) * 1000) / 1000

        return class_width, class_offset

    def test_empty_series(self):
        class_count       =  100
        x                 =  np.array([])
        x_max             =  1
        x_min             = -1
        class_width, \
         class_offset     =  self.class_param(x, class_count)
        hysteresis        =  class_width
        enforce_margin    =  0  # First and last data point may be excluded in tp
        use_HCM           =  0  # Use 4 point method, not HCM
        use_ASTM          =  0  # Use 4 point method, not ASTM
        residual_method   =  0  # No processing on residue
        spread_damage     =  0  # No damage spreading

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        self.assertEqual(res["rfm"].sum(), 0)
        self.assertEqual(len(res["res"]), 0)

    def test_single_cycle_up(self):
        class_count       =  4
        x                 =  np.array([1, 3, 2, 4])
        x_max             =  4
        x_min             =  1
        class_width, \
         class_offset     =  self.class_param(x, class_count)
        hysteresis        =  class_width * 0.99
        enforce_margin    =  0  # First and last data point may be excluded in tp
        use_HCM           =  0  # Use 4 point method, not HCM
        use_ASTM          =  0  # Use 4 point method, not ASTM
        residual_method   =  0  # No processing on residue
        spread_damage     =  0  # No damage spreading

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        self.assertEqual(res["rfm"].sum(), 1)
        self.assertEqual(res["rfm"][3 - 1, 2 - 1], 1)
        self.assertTrue((res["res"].flatten() == [1, 4]).all())

    def test_one_cycle_down(self):
        class_count       =  4
        x                 =  np.array([4, 2, 3, 1])
        x_max             =  4
        x_min             =  1
        class_width, \
         class_offset     =  self.class_param(x, class_count)
        hysteresis        =  class_width * 0.99
        enforce_margin    =  0  # First and last data point may be excluded in tp
        use_HCM           =  0  # Use 4 point method, not HCM
        use_ASTM          =  0  # Use 4 point method, not ASTM
        residual_method   =  0  # No processing on residue
        spread_damage     =  0  # No damage spreading

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        self.assertEqual(res["rfm"].sum(), 1)
        self.assertEqual(res["rfm"][2 - 1, 3 - 1], 1)
        self.assertTrue((res["res"].flatten() == [4, 1]).all())

    def test_small_sample(self):
        class_count       =  6
        x                 =  np.array([2, 5, 3, 6, 2, 4, 1, 6, 1,
                                       4, 1, 5, 3, 6, 3, 6, 1, 5, 2])
        x_max             =  x.max()
        x_min             =  x.min()
        class_width, \
         class_offset     =  self.class_param(x, class_count)
        hysteresis        =  class_width
        enforce_margin    =  0  # First and last data point may be excluded in tp
        use_HCM           =  0  # Use 4 point method, not HCM
        use_ASTM          =  0  # Use 4 point method, not ASTM
        residual_method   =  0  # No processing on residue
        spread_damage     =  0  # No damage spreading

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        self.assertEqual(res["rfm"].sum(), 7)
        self.assertEqual(res["rfm"][5 - 1, 3 - 1], 2)
        self.assertEqual(res["rfm"][6 - 1, 3 - 1], 1)
        self.assertEqual(res["rfm"][1 - 1, 4 - 1], 1)
        self.assertEqual(res["rfm"][2 - 1, 4 - 1], 1)
        self.assertEqual(res["rfm"][1 - 1, 6 - 1], 2)

        self.assertTrue((res["res"].flatten() == [2, 6, 1, 5, 2]).all())

    def test_long_series(self):
        try:
            import pandas as pd
        except ImportError as err:
            print("This test requires module 'pandas'!")
            raise err

        class_count       =  100
        class_offset      = -2025
        class_width       =  50
        x                 =  pd.read_csv(os.path.join(self.get_script_path(), "long_series.csv"), header=None)
        x                 =  x.to_numpy().squeeze()
        hysteresis        =  class_width
        use_HCM           =  0
        enforce_margin    =  1  # Enforce first and last data point included in tp
        use_HCM           =  0  # Use 4 point method, not HCM
        use_ASTM          =  0  # Use 4 point method, not ASTM
        residual_method   =  0  # 0=RFC_RES_NONE, 7=RFC_RES_REPEATED
        spread_damage     =  1  # 0=RFC_SD_HALF_23, 1=RFC_SD_RAMP_AMPLITUDE_23

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        # With residuum:    pd == 9.8934e-06 (repeated)
        # Without residuum: pd == 1.1486e-07
        self.assertTrue(np.absolute(res["tp"][:, 2].sum() / res["damage"] - 1) < 1e-10)

        spread_damage = 8;  # 7=RFC_SD_TRANSIENT_23, 8=RFC_SD_TRANSIENT_23c

        res = rfcnt.rfc(
            x, class_count=class_count,
            class_width=class_width,
            class_offset=class_offset,
            hysteresis=hysteresis,
            residual_method=residual_method,
            enforce_margin=enforce_margin,
            use_HCM=use_HCM,
            use_ASTM=use_ASTM,
            spread_damage=spread_damage)

        #assert(abs(sum(dh) / pd - 1) < 1e-10)

        self.assertEqual("%.4e" % res["damage"], "1.1486e-07")
        self.assertEqual(res["rfm"].sum(), 640)
        self.assertEqual(len(res["res"]), 10)
        test = np.absolute(res["res"].flatten() - [
                           0, 142, -609, 2950, -2000,
                           2159, 1894, 2101, 1991, 2061])
        self.assertTrue(test.sum() < 1e-3)


def run():
    unittest.main()
